import 'package:test/test.dart';
import 'package:yak_runner/yak_runner.dart';
import '../../mocks/all.dart';

void main() {
  const data = 1;
  const res = '$data';

  group('`onResult` MIXIN', () {
    final exceptionHandler = MockExceptionHandler();
    final firstDelegate = MockDelegate<int>();

    final secondDelegate = MockUnaryDelegate<String, int>();
    final firstRunner = YakRunner<int>(
      firstDelegate,
      handleException: exceptionHandler,
    );

    final secondRunner = YakRunnerArg<String, int>(
      secondDelegate,
      handleException: exceptionHandler,
    );
    test('WHEN `Delegate<S>` fails THEN  `onResult() return Failure<T>', () {
      exceptionHandler.reset;
      firstDelegate.reset;
      secondDelegate.reset;

      firstDelegate.result = () => throw Exception();
      secondDelegate.result = () => res;
      exceptionHandler.result = () {};

      final result = firstRunner().onResult<String>(secondRunner);

      expect(
        result,
        isNotNull,
        reason: '`result` must not be null',
      );
      expect(
        result,
        isNot(Success),
        reason: '`result` should not be `Success`',
      );
      expect(
        result is Failure<int>,
        false,
        reason: '`result`should not be `Failure<S>`',
      );
      expect(
        result,
        isA<Failure<String>>(),
        reason: '`result` should be `Failure<T>`',
      );
      expect(
        firstDelegate.callCount,
        1,
        reason: '`firstDelegate` should be called once',
      );
      expect(
        secondDelegate.callCount,
        0,
        reason: '`secondDelegate` should NOT be called',
      );
      expect(
        exceptionHandler.callCount,
        1,
        reason: '`exceptionHandler` shoul be called once',
      );
    });

    test('WHEN `ArgDelegate<T,S>` fail THEN  `onResult() return Failure<T>',
        () {
      exceptionHandler.reset;
      firstDelegate.reset;
      secondDelegate.reset;

      firstDelegate.result = () => data;
      secondDelegate.result = () => throw Exception();
      exceptionHandler.result = () {};

      final result = firstRunner().onResult<String>(secondRunner);

      expect(
        result,
        isNotNull,
        reason: '`result` must not be null',
      );
      expect(
        result,
        isNot(Success),
        reason: '`result` should not be `Success`',
      );
      expect(
        result is Failure<int>,
        false,
        reason: '`result`should not be `Failure<S>`',
      );
      expect(
        result,
        isA<Failure<String>>(),
        reason: '`result` should be `Failure<T>`',
      );
      expect(
        firstDelegate.callCount,
        1,
        reason: '`firstDelegate` should be called once',
      );
      expect(
        secondDelegate.callCount,
        1,
        reason: '`secondDelegate` should NOT be called',
      );
      expect(
        exceptionHandler.callCount,
        1,
        reason: '`exceptionHandler` shoul be called once',
      );
    });
  });
}
